"use strict";
var alsatian_core_1 = require("../alsatian-core");
var create_promise_1 = require("../../promise/create-promise");
var _errors_1 = require("../_errors");
var TestItem = (function () {
    function TestItem(_testFixture, _test, _testCase) {
        this._testFixture = _testFixture;
        this._test = _test;
        this._testCase = _testCase;
    }
    Object.defineProperty(TestItem.prototype, "testCase", {
        get: function () {
            return this._testCase;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TestItem.prototype, "test", {
        get: function () {
            return this._test;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TestItem.prototype, "testFixture", {
        get: function () {
            return this._testFixture;
        },
        enumerable: true,
        configurable: true
    });
    TestItem.prototype.run = function (timeout) {
        var _this = this;
        var promise = create_promise_1.createPromise();
        setTimeout(function () { return _this._run(promise, timeout); });
        return promise;
    };
    TestItem.prototype._run = function (promise, timeout) {
        if (this._test.ignored) {
            promise.resolve({ test: this._test });
        }
        else {
            this._setup();
            if (this._test.isAsync) {
                this._runAsync(promise, timeout);
            }
            else {
                this._runSync(promise);
            }
        }
    };
    TestItem.prototype._runSync = function (promise) {
        try {
            this._testFixture.fixture[this._test.key].apply(this._testFixture.fixture, this._testCase.arguments);
            this._tearDown();
            promise.resolve({ test: this._test });
        }
        catch (error) {
            this._tearDown();
            promise.resolve({ test: this._test, error: error });
        }
    };
    TestItem.prototype._runAsync = function (promise, timeout) {
        var _this = this;
        var timeoutExpired = false;
        var testPromise = this._testFixture.fixture[this._test.key].apply(this._testFixture.fixture, this._testCase.arguments);
        var timeoutCheck = null;
        testPromise.then(function () {
            if (!timeoutExpired) {
                clearTimeout(timeoutCheck);
                _this._tearDown();
                promise.resolve({ test: _this._test });
            }
        })
            .catch(function (error) {
            _this._tearDown();
            promise.resolve({ test: _this._test, error: error });
        });
        var testTimeout = this._test.timeout || timeout;
        timeoutCheck = setTimeout(function () {
            timeoutExpired = true;
            var error = new _errors_1.TestTimeoutError(testTimeout);
            _this._tearDown();
            promise.resolve({ test: _this._test, error: error });
        }, testTimeout);
    };
    TestItem.prototype._reportResult = function (promise, error) {
        this._tearDown();
        promise.resolve({ test: this._test, error: error });
    };
    TestItem.prototype._setup = function () {
        var _this = this;
        var setupFunctions = Reflect.getMetadata(alsatian_core_1.METADATA_KEYS.SETUP, this._testFixture.fixture);
        if (setupFunctions) {
            setupFunctions.forEach(function (setupFunction) {
                _this._testFixture.fixture[setupFunction].call(_this._testFixture.fixture);
            });
        }
    };
    TestItem.prototype._tearDown = function () {
        var _this = this;
        var teardownFunctions = Reflect.getMetadata(alsatian_core_1.METADATA_KEYS.TEARDOWN, this._testFixture.fixture);
        if (teardownFunctions) {
            teardownFunctions.forEach(function (teardownFunction) {
                _this._testFixture.fixture[teardownFunction].call(_this._testFixture.fixture);
            });
        }
    };
    return TestItem;
}());
exports.TestItem = TestItem;
//# sourceMappingURL=test-item.js.map