"use strict";
var create_promise_1 = require("../../promise/create-promise");
var alsatian_core_1 = require("../alsatian-core");
var test_plan_1 = require("./test-plan");
var test_set_run_info_1 = require("./test-set-run-info");
require("reflect-metadata");
var TestRunner = (function () {
    function TestRunner(output) {
        // If we were given a TestOutput, use it, otherwise make one
        if (output !== undefined) {
            this._output = output;
        }
        else {
            this._output = new alsatian_core_1.TestOutput(process.stdout);
        }
    }
    TestRunner.prototype.run = function (testSet, timeout) {
        var testPlan = new test_plan_1.TestPlan(testSet);
        if (testPlan.testItems.length === 0) {
            throw new Error("no tests to run.");
        }
        if (!timeout) {
            timeout = 500;
        }
        var testSetResults = new alsatian_core_1.TestSetResults();
        this._output.emitVersion();
        this._output.emitPlan(testPlan.testItems.length);
        var promise = create_promise_1.createPromise();
        var testSetRunInfo = new test_set_run_info_1.TestSetRunInfo(promise, testPlan, testSetResults, timeout);
        this._scheduleNextTestPlanItem(testSetRunInfo);
        return promise;
    };
    TestRunner.prototype._createResultAndRunNextTest = function (testSetRunInfo, error) {
        var testSetResults = testSetRunInfo.testSetResults;
        var currentTestFixtureResults = testSetResults
            .testFixtureResults[testSetResults.testFixtureResults.length - 1];
        var currentTestResults = currentTestFixtureResults
            .testResults[currentTestFixtureResults.testResults.length - 1];
        var testItem = testSetRunInfo.testPlanItem;
        var result = currentTestResults.addTestCaseResult(testItem.testCase.arguments, error);
        this._output.emitResult(testSetRunInfo.testPlan.testItems.indexOf(testItem) + 1, result);
        this._scheduleNextTestPlanItem(testSetRunInfo);
    };
    TestRunner.prototype._scheduleNextTestPlanItem = function (testSetRunInfo) {
        var _this = this;
        var nextTestPlanIndex = testSetRunInfo.testPlan.testItems.indexOf(testSetRunInfo.testPlanItem) + 1;
        var nextTestPlanItem = testSetRunInfo.testPlan.testItems[nextTestPlanIndex];
        if (nextTestPlanItem) {
            testSetRunInfo.testPlanItem = nextTestPlanItem;
            var testSetResults = testSetRunInfo.testSetResults;
            var currentTestFixtureResults = testSetResults
                .testFixtureResults[testSetResults.testFixtureResults.length - 1];
            if (!currentTestFixtureResults || currentTestFixtureResults.fixture !== nextTestPlanItem.testFixture) {
                this._output.emitFixture(nextTestPlanItem.testFixture);
                currentTestFixtureResults = testSetResults.addTestFixtureResult(nextTestPlanItem.testFixture);
            }
            var currentTestResults = currentTestFixtureResults
                .testResults[currentTestFixtureResults.testResults.length - 1];
            if (!currentTestResults || currentTestResults.test !== nextTestPlanItem.test) {
                currentTestResults = currentTestFixtureResults.addTestResult(nextTestPlanItem.test);
            }
            nextTestPlanItem.run(testSetRunInfo.timeout)
                .then(function (testResults) {
                _this._createResultAndRunNextTest(testSetRunInfo, testResults.error);
            })
                .catch(function (error) {
                console.log(error);
            });
        }
        else {
            testSetRunInfo.promise.resolve(testSetRunInfo.testSetResults);
        }
    };
    ;
    return TestRunner;
}());
exports.TestRunner = TestRunner;
//# sourceMappingURL=test-runner.js.map