"use strict";
var create_promise_1 = require("../promise/create-promise");
var alsatian_core_1 = require("./alsatian-core");
require("reflect-metadata");
var TestRunner = (function () {
    function TestRunner(output) {
        var _this = this;
        this._currentTestId = 0;
        this._currentTestFixtureIndex = 0;
        this._currentTestIndex = 0;
        this._currentTestCaseIndex = 0;
        this._resultPromise = create_promise_1.createPromise();
        this._testResults = new alsatian_core_1.TestSetResults();
        this._testTimeout = 500;
        this._runNextTestCase = function () {
            _this._currentTestCaseIndex++;
            if (!_this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex].testCases[_this._currentTestCaseIndex]) {
                _this._runNextTest();
            }
            else {
                _this._runTest(_this._testFixtures[_this._currentTestFixtureIndex], _this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex], _this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex].testCases[_this._currentTestCaseIndex].arguments);
            }
        };
        // If we were given a TestOutput, use it, otherwise make one
        if (output !== undefined) {
            this._output = output;
        }
        else {
            this._output = new alsatian_core_1.TestOutput(process.stdout);
        }
    }
    TestRunner.prototype.run = function (testSet, timeout) {
        var _this = this;
        if (timeout) {
            this._testTimeout = timeout;
        }
        var anyTestsFocussed = testSet.testFixtures.filter(function (testFixture) { return testFixture.focussed || testFixture.getTests().filter(function (test) { return test.focussed; }).length > 0; }).length > 0;
        this._testFixtures = testSet.testFixtures;
        // Filter out unfocussed tests if any are focussed
        if (anyTestsFocussed) {
            this._testFixtures = testSet.testFixtures.filter(function (testFixture) { return testFixture.focussed || testFixture.getTests().filter(function (test) { return test.focussed; }).length > 0; });
        }
        var totalTestCount = this._testFixtures
            .filter(function (x) { return x.getTests().length > 0; })
            .map(function (x) { return x.getTests().map(function (y) { return y.testCases.length; })
            .reduce(function (a, b) { return a + b; }, 0); })
            .reduce(function (a, b) { return a + b; }, 0);
        if (totalTestCount === 0) {
            throw new Error("no tests to run.");
        }
        else {
            this._output.emitVersion();
            this._output.emitPlan(totalTestCount);
            this._currentTestFixtureResults = this._testResults.addTestFixtureResult(this._testFixtures[this._currentTestFixtureIndex]);
            this._currentTestResults = this._currentTestFixtureResults.addTestResult(this._testFixtures[this._currentTestFixtureIndex].getTests()[this._currentTestIndex]);
            setTimeout(function () {
                _this._runTest(_this._testFixtures[_this._currentTestFixtureIndex], _this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex], _this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex].testCases[_this._currentTestCaseIndex].arguments);
            });
        }
        return this._resultPromise;
    };
    TestRunner.prototype._runTest = function (testFixture, test, testCaseArguments) {
        var _this = this;
        this._currentTestId++;
        if (test.ignored) {
            var result = this._currentTestResults.addTestCaseResult(testCaseArguments, undefined);
            this._output.emitResult(this._currentTestId, result);
            this._runNextTest();
            return;
        }
        var setupFunctions = Reflect.getMetadata(alsatian_core_1.METADATA_KEYS.SETUP, testFixture.fixture);
        if (setupFunctions) {
            setupFunctions.forEach(function (setupFunction) {
                testFixture.fixture[setupFunction].call(testFixture.fixture);
            });
        }
        try {
            if (test.isAsync) {
                var timeout_1 = false;
                var promise = testFixture.fixture[test.key].apply(testFixture.fixture, testCaseArguments);
                var timeoutCheck_1 = null;
                promise.then(function () {
                    if (!timeout_1) {
                        _this._notifySuccess(test, testCaseArguments);
                        clearTimeout(timeoutCheck_1);
                    }
                })
                    .catch(function (error) {
                    _this._handleError(error, test, testCaseArguments);
                });
                timeoutCheck_1 = setTimeout(function () {
                    timeout_1 = true;
                    _this._handleError(new alsatian_core_1.TestTimeoutError(test.timeout || _this._testTimeout), test, testCaseArguments);
                }, test.timeout || this._testTimeout);
            }
            else {
                testFixture.fixture[test.key].apply(testFixture, testCaseArguments);
                this._notifySuccess(test, testCaseArguments);
            }
        }
        catch (error) {
            this._handleError(error, test, testCaseArguments);
        }
    };
    TestRunner.prototype._handleError = function (error, test, testCaseArguments) {
        this._teardown();
        var result = this._currentTestResults.addTestCaseResult(testCaseArguments, error);
        this._output.emitResult(this._currentTestId, result);
        this._runNextTestCase();
    };
    TestRunner.prototype._notifySuccess = function (test, testCaseArguments) {
        this._teardown();
        var result = this._currentTestResults.addTestCaseResult(testCaseArguments);
        this._output.emitResult(this._currentTestId, result);
        this._runNextTestCase();
    };
    TestRunner.prototype._teardown = function () {
        var testFixture = this._testFixtures[this._currentTestFixtureIndex];
        var teardownFunctions = Reflect.getMetadata(alsatian_core_1.METADATA_KEYS.TEARDOWN, testFixture.fixture);
        if (teardownFunctions) {
            teardownFunctions.forEach(function (teardownFunction) {
                testFixture.fixture[teardownFunction].call(testFixture.fixture);
            });
        }
    };
    TestRunner.prototype._exit = function () {
        this._resultPromise.resolve(this._testResults);
    };
    TestRunner.prototype._runNextTestFixture = function () {
        var _this = this;
        this._currentTestFixtureIndex++;
        this._currentTestIndex = 0;
        this._currentTestCaseIndex = 0;
        // no more test fixtures
        if (!this._testFixtures[this._currentTestFixtureIndex]) {
            this._exit();
        }
        else if (!this._testFixtures[this._currentTestFixtureIndex].getTests()[this._currentTestIndex]) {
            this._runNextTestFixture();
        }
        else {
            this._currentTestFixtureResults = this._testResults.addTestFixtureResult(this._testFixtures[this._currentTestFixtureIndex]);
            setTimeout(function () {
                _this._runTest(_this._testFixtures[_this._currentTestFixtureIndex], _this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex], _this._testFixtures[_this._currentTestFixtureIndex].getTests()[_this._currentTestIndex].testCases[_this._currentTestCaseIndex].arguments);
            });
        }
    };
    TestRunner.prototype._runNextTest = function () {
        this._currentTestIndex++;
        this._currentTestCaseIndex = 0;
        if (!this._testFixtures[this._currentTestFixtureIndex].getTests()[this._currentTestIndex]) {
            this._runNextTestFixture();
        }
        else {
            this._currentTestResults = this._currentTestFixtureResults.addTestResult(this._testFixtures[this._currentTestFixtureIndex].getTests()[this._currentTestIndex]);
            this._runTest(this._testFixtures[this._currentTestFixtureIndex], this._testFixtures[this._currentTestFixtureIndex].getTests()[this._currentTestIndex], this._testFixtures[this._currentTestFixtureIndex].getTests()[this._currentTestIndex].testCases[this._currentTestCaseIndex].arguments);
        }
    };
    return TestRunner;
}());
exports.TestRunner = TestRunner;
//# sourceMappingURL=test-runner.js.map